"""
##################################################################################
#
# Website As App
# Run any website as standalone desktop application
#
# @author    Marcin Orlowski <mail (#) marcinOrlowski (.) com>
# @copyright 2023-2024 Marcin Orlowski
# @license   https://www.opensource.org/licenses/mit-license.php MIT
# @link      https://github.com/MarcinOrlowski/website-as-app
#
##################################################################################
"""
import importlib.resources as pkg_resources
import os, re
from typing import Optional

import argparse
from PySide6.QtGui import QIcon


class Utils(object):
    @staticmethod
    def get_icon(icon: Optional[str] = None) -> QIcon:
        """
        Attempts to construct QIcon object from given icon file path. If icon file is not given
        or does not exist, default icon is used (so it always return valid QIcon object).

        :param icon: Path to icon file (in format supported by QT, i.e. PNG)
        :return: QIcon object
        """
        icon_file = None

        if icon and os.path.exists(icon):
            icon_file = icon

        if icon_file is None:
            with pkg_resources.path('websiteapp.icons', 'default.png') as icon_path:
                icon_file = str(icon_path)

        return QIcon(icon_file)

    @staticmethod
    def parse_geometry(geometry_string: str) -> (int, int, int, int):
        """
        Parse the geometry string and return the width, height, x, and y values.

        :param geometry_string: A string representing the geometry in the format WIDTHxHEIGHT+X+Y.

        :return: A tuple containing the x, y,  width and height values.
        :raises ValueError: If the geometry string format is incorrect.
        """
        match = re.search(r'^(\d+)x(\d+)\+(\d+)\+(\d+)$', geometry_string)
        if not match:
            raise ValueError(f"Invalid geometry. Expected WIDTHxHEIGHT+X+Y, got '{geometry_string}")

        width = int(match.group(1))
        if width < 1:
            raise ValueError(f"Invalid geometry. Width must be greater than 0, got '{width}'")
        height = int(match.group(2))
        if height < 1:
            raise ValueError(f"Invalid geometry. Height must be greater than 0, got '{height}'")
        x = int(match.group(3))
        if x < 0:
            raise ValueError(f"Invalid geometry. X must be greater than or equal to 0, got '{x}'")
        y = int(match.group(4))
        if y < 0:
            raise ValueError(f"Invalid geometry. Y must be greater than or equal to 0, got '{y}'")

        return x, y, width, height

    @staticmethod
    def handle_args():
        """
        Create an argument parser to handle command line arguments for opening a website in a
        standalone window.

        :return: The parsed command line arguments.
        """
        parser = argparse.ArgumentParser(
            description="Open any website in standalone window (like it's an app)")
        parser.add_argument('url', type=str, help='The URL to open')

        parser.add_argument('--profile', '-p', type=str, default='default',
                            help='Profile name (for cookies isolation etc). Default: "%(default)s"')

        # Can't use "title" as it is swallowed by QT and used for window title which cannot be later
        # changed. So we use "name" instead.
        parser.add_argument('--name', '-n', type=str, default=None,
                            help='Application name (shown as window title)')

        parser.add_argument('--icon', '-i', type=str, default=None,
                            help='Full path to PNG image file to be used as app icon')
        parser.add_argument('--geometry', '-g', type=str, default='450x600+0+0',
                            help='Initial window geometry ("WIDTHxHEIGHT+X+Y"). Default: "%(default)s"')
        parser.add_argument('--zoom', '-z', type=float, default="1.0",
                            help='WebView zoom factor. Default: %(default)s')
        parser.add_argument('--no-tray', action='store_true',
                            help='Disables docking app in system tray')
        # parser.add_argument('--minimized', '-m', action='store_true',
        #                     help='Starts app minimized', target='minimized')

        parser.add_argument('--debug', '-d', action='store_true',
                            help='Makes app print more debug messages during execution')

        return parser.parse_args()



"""
##################################################################################
#
# Website As App
# Run any website as standalone desktop application
#
# @author    Marcin Orlowski <mail (#) marcinOrlowski (.) com>
# @copyright 2023-2024 Marcin Orlowski
# @license   https://www.opensource.org/licenses/mit-license.php MIT
# @link      https://github.com/MarcinOrlowski/website-as-app
#
#####
"""

from typing import List


class Const(object):
    APP_NAME: str = 'Website As App'
    APP_PROJECT_NAME: str = 'website-as-app'
    APP_VERSION: str = '1.0.0'
    APP_URL: str = 'https://github.com/MarcinOrlowski/website-as-app/'
    APP_DESCRIPTION: str = 'Opens any web site as standalone desktop app.'
    APP_YEAR: int = 2024

    APP_DESCRIPTION: List[str] = [
        f'{APP_NAME} v{APP_VERSION} * Copyright 2023-{APP_YEAR} by Marcin Orlowski.',
        APP_DESCRIPTION,
        APP_URL,
    ]



"""
##################################################################################
#
# Website As App
# Run any website as standalone desktop application
#
# @author    Marcin Orlowski <mail (#) marcinOrlowski (.) com>
# @copyright 2023-2024 Marcin Orlowski
# @license   https://www.opensource.org/licenses/mit-license.php MIT
# @link      https://github.com/MarcinOrlowski/website-as-app
#
##################################################################################
"""
import webbrowser
import importlib.resources as pkg_resources

from PySide6.QtCore import Qt
from PySide6.QtGui import QPixmap
from PySide6.QtWidgets import QApplication, QVBoxLayout, QLabel, QDialog, QPushButton, QHBoxLayout

from websiteapp.const import Const


class About(QDialog):
    def __init__(self):
        super().__init__()

        self.app = QApplication.instance()

        # Set the layout for the dialog
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 6)
        layout.setSpacing(0)
        self.setLayout(layout)
        self.setWindowTitle(f"About {Const.APP_NAME}")

        # Create a label for the logo image
        logo_label = QLabel(self)
        with pkg_resources.path('websiteapp.icons', 'logo.png') as icon_path:
            icon_file = str(icon_path)
        logo_label.setPixmap(QPixmap(icon_file))
        layout.addWidget(logo_label)

        # Create a label for the credits text
        label_layout = QVBoxLayout()

        label = QLabel(f'<b>{Const.APP_NAME}</b> v{Const.APP_VERSION}', self)
        label.setTextFormat(Qt.RichText)
        label.setAlignment(Qt.AlignCenter)
        font = label.font()
        font.setPointSize(font.pointSize() * 1.7)
        label.setFont(font)
        label_layout.addWidget(label)

        label = QLabel(f'&copy;2023-{Const.APP_YEAR} Marcin Orlowski', self)
        label.setTextFormat(Qt.RichText)
        label.setAlignment(Qt.AlignCenter)
        font = label.font()
        font.setPointSize(font.pointSize() * 1.2)
        label.setFont(font)
        label_layout.addWidget(label)

        label = QLabel(Const.APP_URL, self)
        # label.setTextFormat(Qt.RichText)
        label.setAlignment(Qt.AlignCenter)
        label_layout.addWidget(label)

        layout.addLayout(label_layout)

        # # Create a horizontal layout for the buttons
        bt_website = QPushButton("Open project website", self)
        bt_website.clicked.connect(self.on_ok_clicked)
        bt_close = QPushButton("Close", self)
        bt_close.clicked.connect(self.on_cancel_clicked)
        bt_close.setFocus()

        button_layout = QHBoxLayout()
        button_layout.setContentsMargins(6, 6, 6, 6)
        button_layout.setSpacing(6)
        button_layout.addWidget(bt_website)
        button_layout.addWidget(bt_close)
        layout.addLayout(button_layout)

    def on_ok_clicked(self):
        webbrowser.open(Const.APP_URL)
        self.reject()

    def on_cancel_clicked(self):
        self.reject()



"""
##################################################################################
#
# Website As App
# Run any website as standalone desktop application
#
# @author    Marcin Orlowski <mail (#) marcinOrlowski (.) com>
# @copyright 2023-2024 Marcin Orlowski
# @license   https://www.opensource.org/licenses/mit-license.php MIT
# @link      https://github.com/MarcinOrlowski/website-as-app
#
#####
"""



"""
##################################################################################
#
# Website As App
# Run any website as standalone desktop application
#
# @author    Marcin Orlowski <mail (#) marcinOrlowski (.) com>
# @copyright 2023-2024 Marcin Orlowski
# @license   https://www.opensource.org/licenses/mit-license.php MIT
# @link      https://github.com/MarcinOrlowski/website-as-app
#
##################################################################################
"""
import sys
from typing import Optional

from PySide6.QtCore import QUrl
from PySide6.QtGui import QIcon, QAction
from PySide6.QtWebEngineCore import QWebEngineProfile, QWebEnginePage, QWebEngineSettings
from PySide6.QtWebEngineWidgets import QWebEngineView
from PySide6.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QWidget, QSystemTrayIcon, \
    QMenu

from websiteapp.about import About
from websiteapp.const import Const
from websiteapp.utils import Utils


class WebApp(QMainWindow):
    about_dialog: Optional[About] = None

    def __init__(self):
        super().__init__()

        self.app = QApplication.instance()
        self.args = Utils.handle_args()

        # Set window geometry
        x, y, width, height = Utils.parse_geometry(self.args.geometry)
        self.setGeometry(x, y, width, height)
        self.dbug(f'Geometry: {width}x{height}+{x}+{y}')

        app_icon = Utils.get_icon(self.args.icon)
        self.setWindowIcon(app_icon)
        if self.args.no_tray:
            self.app.setQuitOnLastWindowClosed(True)  # Ensure we quit when last window is closed
        else:
            self.setup_tray_icon(app_icon)

        window_title = self.args.name if self.args.name else self.args.url
        window_title += f' ({self.args.profile})'
        self.setWindowTitle(f'{window_title} · {Const.APP_NAME}')

        # Create a persistent profile (cookie jar etc.)
        self.dbug(f'Profile: {self.args.profile}')
        self.profile = QWebEngineProfile(self.args.profile, self)
        self.page = QWebEnginePage(self.profile, self)

        self.browser = QWebEngineView(self)
        web_settings = self.browser.settings()
        web_settings.setAttribute(
            QWebEngineSettings.WebAttribute.JavascriptCanAccessClipboard, True)

        self.browser.setPage(self.page)
        self.browser.setZoomFactor(self.args.zoom)

        self.dbug(f'URL: {self.args.url}')
        self.browser.setUrl(QUrl(self.args.url))

        # Window layout
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        layout.addWidget(self.browser)

        central_widget = QWidget()
        central_widget.setLayout(layout)
        self.setCentralWidget(central_widget)

    def closeEvent(self, event) -> None:
        """
        Override closeEvent to hide the window instead of closing it and hides the window instead.

        :param event:
        """
        if not self.args.no_tray:
            self.hide()
            event.ignore()
        else:
            super().closeEvent(event)

    def setup_tray_icon(self, icon) -> None:
        """
        This method is used to set up the system tray icon for the application.

        :param icon: QPixmap object representing the icon that will be used in the system tray.
        """
        tray_menu = QMenu()

        tray_icon = QSystemTrayIcon(icon, self.app)
        tray_icon.setContextMenu(tray_menu)
        tray_icon.activated.connect(self.on_tray_icon_activated)

        about_label = f'About {Const.APP_NAME}'
        about_action = QAction(about_label, self.app)
        about_action.triggered.connect(self.open_about_dialog)
        tray_menu.addAction(about_action)

        quit_label = f'Quit {self.args.name}' if self.args.name else 'Quit'
        quit_action = QAction(quit_label, self.app)
        quit_action.triggered.connect(self.quit_app)
        tray_menu.addAction(quit_action)

        tray_icon.show()

    def quit_app(self) -> None:
        """
        Closes the application.
        """
        self.app.quit()

    def open_about_dialog(self, checked) -> None:
        if not self.about_dialog:
            self.about_dialog = About()

        if not self.about_dialog.isVisible():
            self.about_dialog.show()
        self.about_dialog.activateWindow()

    def on_tray_icon_activated(self, reason) -> None:
        """
        Callback invoked when application icon in system tray is clicked.
        """
        if reason == QSystemTrayIcon.ActivationReason.Trigger:
            # Check for left-click (Trigger)
            self.toggle_window()

    def toggle_window(self) -> None:
        """
        Toggles the visibility of the window.
        """
        self.hide() if self.isVisible() else self.show()

    # ############################################################################################ #

    def dbug(self, msg: str) -> None:
        if self.args.debug:
            print(msg, file=sys.stderr)

    # ############################################################################################ #

    @staticmethod
    def run() -> None:
        """
        Application entry point. When renamed, ensure setup.py's reference is updated as well.
        """
        app = QApplication(sys.argv)
        app.setOrganizationName("MarcinOrlowski")
        app.setApplicationName("Website As App")

        window = WebApp()
        window.show()

        app.exec()



#!/usr/bin/env python3

"""
##################################################################################
#
# Website As App
# Run any website as standalone desktop application
#
# @author    Marcin Orlowski <mail (#) marcinOrlowski (.) com>
# @copyright 2023-2024 Marcin Orlowski
# @license   https://www.opensource.org/licenses/mit-license.php MIT
# @link      https://github.com/MarcinOrlowski/website-as-app
#
#####
#
# https://blog.ganssle.io/articles/2021/10/setup-py-deprecated.html
#
# python -m venv venv
# source venv/bin/activate
# pip install -r requirements-dev.txt
# python -m build
# # Reinstall the app (do not do "install --upgrade" as cached bytecode can not be updated)
# pip uninstall --yes dist/website_as_app-1.0.0-py3-none-any.whl
# # intentionally no --upgrade for install to endforce conflict if not uninstalled fully first.
# pip install dist/website_as_app-1.0.0-py3-none-any.whl
# twine upload dist/*
#
"""

from setuptools import setup, find_packages

from websiteapp.const import Const

with open('README.md', 'r') as fh:
    readme = fh.read()

setup(
    name=Const.APP_PROJECT_NAME,
    version=Const.APP_VERSION,
    packages=find_packages(),

    install_requires=[
        'argparse>=1.4.0',
        'PySide6',
        'PyQtWebEngine',
    ],
    entry_points={
        'console_scripts': [
            'webapp = websiteapp.webapp:WebApp.run',
            'runasapp = websiteapp.webapp:WebApp.run',
        ],
    },

    package_data={
        'websiteapp': [
            'icons/default.png',
            'icons/logo.png',
        ],
    },

    author='Marcin Orlowski',
    author_email='mail@marcinOrlowski.com',
    description=Const.APP_DESCRIPTION,
    long_description=readme,
    long_description_content_type='text/markdown',
    url=Const.APP_URL,
    keywords='webapp desktop app',
    project_urls={
        'Bug Tracker': f'{Const.APP_URL}/issues/',
        'Documentation': Const.APP_URL,
        'Source Code': Const.APP_URL,
    },
    # https://choosealicense.com/
    license='MIT License',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)
